// Functions for easy use of KAmod module with MMA8653FC accelerometer.
// jk 07'2018

#ifndef	_KAMOD_MMA8653FC_H
#define	_KAMOD_MMA8653FC_H
#include <stdint.h>
#include "kamod_common.h"

// Possible datarates of accelerometer 
// i.e. possible values for datarate parameter of mma8653fc_init function.
#define MMA8653FC_DATARATE_1HZ56		7
#define MMA8653FC_DATARATE_6HZ25		6
#define MMA8653FC_DATARATE_12HZ5		5
#define MMA8653FC_DATARATE_50HZ			4
#define MMA8653FC_DATARATE_100HZ		3
#define MMA8653FC_DATARATE_200HZ		2
#define MMA8653FC_DATARATE_400HZ		1
#define MMA8653FC_DATARATE_800HZ		0

// Possible fullscale range values of accelerometer 
// i.e. possible values for fullscale parameter of mma8653fc_init function.
// Fullscale range of x means the values range from -x to +x.
#define MMA8653FC_FULLSCALE_2G			0
#define MMA8653FC_FULLSCALE_4G			1
#define MMA8653FC_FULLSCALE_8G			2

// Raw value corresponding to the maximum acceleration. 
#define MMA8653FC_FULLSCALE_VAL			0x8000u

/* Initialize MMA8653FC accelerometer for use with KAmod connector. 
 * The function should be called before using any other MMA8653FC function. 
 * It also checks if communication with the sensor is possible and initializes 
 * the sensor itself.  
 * It initializes GPIO lines and I2C interface used to communicate with the sensor. 
 * param datarate: rate of measurements. Use one of the definitions above.
 * param acc_fullscale: full-scale range. Use one of the definitions above.
 * @return 0 on succes, non-zero value if the function was called with incorrect 
 * parameters or interfaceing with the sensor failed. */
int32_t mma8653fc_init(uint8_t datarate, uint8_t fullscale);

/* Read accelerometer measurement of all 3 axes in raw format. 
 * The results are put in a d3_res16 structure. 
 * @param res: pointer to d3_res16 structure to put results in. 
 * @return 0 on succes, non-zero value if reading from the sensor failed */
int32_t mma8653fc_read_xyz(struct d3_res16* res);//(struct mma8653fc_res* res);

/* Read raw x-axis acceleration.  
 * @return x-axis acceleration */
int16_t mma8653fc_read_x(void);

/* Read raw y-axis acceleration.  
 * @return y-axis acceleration */
int16_t mma8653fc_read_y(void);

/* Read raw z-axis acceleration.  
 * @return z-axis acceleration */
int16_t mma8653fc_read_z(void);

/* Read accelerometer measurement of all 3 axes as floating point numbers. 
 * The results are put in a d3_resf structure. 
 * @param res: pointer to d3_resf structure to put results in. 
 * @return 0 on succes, non-zero value if reading from the sensor failed */
int32_t mma8653fc_readf_xyz(struct d3_resf* res);

/* Read x-axis acceleration in multiples of g.  
 * @return x-axis acceleration */
float mma8653fc__readf_x(void);

/* Read y-axis acceleration in multiples of g.  
 * @return y-axis acceleration */
float mma8653fc_readf_y(void);

/* Read z-axis acceleration in multiples of g.  
 * @return z-axis acceleration */
float mma8653fc_readf_z(void);

/* Convert a raw result to a floating point result in multiples of g. 
 * @return result in multiples of g */
float mma8653fc_res_to_float(int16_t res);

/* Get information about the orientation of the sensor. 
 * Orientation result consists of 2 pieces of information:
 * - whether the sensor is facing up (front) or down (back), which can always be determined,
 * - how is the sensor roteted, which can be determined only if the sensor is in vertical position.
 * @param res: raw result. 
 * @return 0 on succes, non-zero value if reading from the sensor failed */
int32_t mma8653fc_read_orientation(struct orientation_res* res);

#endif
